@@ -16,12 +16,13 @@ setup(
         'Programming Language :: Python :: 3.6',
         'Programming Language :: Python :: 3.7',
         'Programming Language :: Python :: 3.8',
+        'Programming Language :: Python :: 3.9',
         'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
         "Operating System :: OS Independent",
     ],
    packages=['behatrix'],  #same as name
    install_requires=[
-          "pyqt5",
-          "numpy",
+          "pyqt5==5.15.2",
+          "numpy==1.19.5",
       ],
  )
