@@ -1,12 +1,8 @@
-#!/usr/bin/env python3
-
 """
 Behatrix
-Behavioral Sequences Analysis (BSA)).
-
 Behavioral sequences analysis with permutations test
 
-Copyright 2017-2019 Olivier Friard
+Copyright 2017-2022 Olivier Friard
 
 This file is part of Behatrix.
 
@@ -25,5 +21,5 @@ This file is part of Behatrix.
 
 """
 
-__version__ = "0.9.9"
-__version_date__ = "2020-03-03"
+__version__ = "0.9.15"
+__version_date__ = "2022-02-10"
